MODULE TRCOMM
  USE TRCOM0
  IMPLICIT NONE

!     ****** CONSTANTS ******
! TRCNS
!        PI    : Pi
!        AEE   : Elementaty charge
!        AME   : Electron mass
!        AMM   : Proton mass
!        VC    : Speed of light in vacuum
!        RMU0  : Permeability of free space
!        EPS0  : Permittivity of free space
!        RKEV  : Factor ([keV] -> [J])
  REAL(8), PARAMETER :: PI   = 3.14159265358979323846D0
  REAL(8), PARAMETER :: AEE  = 1.60217733D-19
  REAL(8), PARAMETER :: AME  = 9.1093897D-31
  REAL(8), PARAMETER :: AMM  = 1.6726231D-27
  REAL(8), PARAMETER :: VC   = 2.99792458D8
  REAL(8), PARAMETER :: RMU0 = 4.D0*PI*1.D-7
  REAL(8), PARAMETER :: EPS0 = 1.D0/(VC*VC*RMU0)
  REAL(8), PARAMETER :: RKEV = AEE*1.D3

!     ****** PARAMETERS ******
! TRPRM
  REAL(8)   :: RR, RA, RKAP, RDLT, BB, RIPS, RIPE, RIPSS, PHIA, PNC, PNFE, PNNU, PNNUS,           &
 &     PROFN1, PROFN2, PROFT1, PROFT2, PROFU1, PROFU2, PROFJ1, PROFJ2, AD0, AV0, CNP, CNH, CDP,   &
 &     CDH, CNN, CWEB, DT, EPSLTR, CHP, CK0, CK1, CKALFA, CKBETA, CKGUMA, CNB, CALF, CSPRS, TSST
  REAL(8), DIMENSION(3) :: ALP
  REAL(8), DIMENSION(8) :: CDW
  REAL(8), DIMENSION(:),ALLOCATABLE :: PA,PZ,PN,PNS,PT,PTS                            ! (NSTM)
  INTEGER(4):: LMAXTR, MDLKAI, MDLETA, MDLAD, MDLAVK, MDLKNC, MDLTPF, NTMAX, NTSTEP, NGTSTP, &
 &     NGRSTP, NGPST, MODELG,MDLDSK,MDTC

!     ****** MODEL PARAMETERS ******
! TRMDL
  REAL(8)   :: TPRST, PNBTOT,PNBR0, PNBRW, PNBVY, PNBVW, PNBENG,PNBRTG
  REAL(8)   :: PECTOT,PECR0, PECRW, PECTOE,PECNPR,PLHTOT,PLHR0, PLHRW, PLHTOE, PLHNPR
  REAL(8)   :: PICTOT,PICR0, PICRW, PICTOE,PICNPR,PNBCD, PECCD, PLHCD, PICCD,  PBSCD
  REAL(8)   :: PELTOT,PELR0, PELRW, PELRAD,PELVEL,PELTIM
  REAL(8), DIMENSION(:),ALLOCATABLE :: PELPAT                                        ! (NSTM)
  INTEGER(4):: MDLNB, MDLEC, MDLLH, MDLIC, MDLCD, MDLPEL, MDLJBS, MDLST, MDLNF, IZERO

!     ****** CONTROL VARIABLES ******
!TRCTL
  REAL(4)   :: GTCPU1
  REAL(8)   :: T, TST, TPRE, WPPRE, RIP, DR, FKAP, RHOA, RIPA, VSEC, RDPS,DIPDT
  REAL(8), DIMENSION(:),ALLOCATABLE :: PNSS                                          ! (NSTM)
  INTEGER(4)::   NT, NRAMAX, NROMAX, NREDGE, NTMAX_SAVE, IREAD

!     ****** MATRIX VARIBALES ******
! TRMTX
  REAL(8), DIMENSION(:,:),ALLOCATABLE :: XV                                  ! (NVM,NRM)
  REAL(8), DIMENSION(:,:),ALLOCATABLE :: YV, AY, Y                           ! (NFM,NRM)
  REAL(8), DIMENSION(:,:),ALLOCATABLE :: ZV, AZ, Z                           ! (NSM,NRM)
  REAL(8), DIMENSION(:,:),ALLOCATABLE :: AX                                  ! (LDAB,MLM)
  REAL(8), DIMENSION(:)  ,ALLOCATABLE :: X                                   ! (MLM)
!
!     ****** FUNDAMENTAL VARIABLES ******
! TRVAR
  REAL(8), DIMENSION(:)  ,ALLOCATABLE :: RG, RM, RHOM, RHOG, BP, RDP, RPSI   ! (NRM)
  REAL(8), DIMENSION(:,:),ALLOCATABLE :: RN, RT, RU                          ! (NRM,NSTM)
  REAL(8), DIMENSION(:,:),ALLOCATABLE :: RW                                  ! (NRM,NFM)

!     ****** PROFILE VARIABLES ******
! TRPFL
  REAL(8), DIMENSION(:,:),ALLOCATABLE :: RNF, RTF                            ! (NRM,NFM)
  REAL(8), DIMENSION(:)  ,ALLOCATABLE :: ANC, ANFE, ANNU, ZEFF, PZC, PZFE, BETA, BETAP, &
 &     BETAL, BETAPL, BETAQ, PBM, PADD, VTOR, VPAR, VPRP, VPOL, WROT, ER, VEXB, WEXB, AGMP   ! (NRM)

!     ****** SOURCE VARIABLES ******
! TRSRC
  REAL(8), DIMENSION(10)      :: RTG
  REAL(8), DIMENSION(:)    ,ALLOCATABLE :: AJ, AJOH, EZOH, QP, AJTOR, AJNB, AJRF, AJBS, PNB, SNB, &
 &     PBIN, PNF, SNF, PFIN, POH, PRL, PCX, PIE, SIE, SCX, TSIE, TSCX        ! (NRM)
  REAL(8), DIMENSION(:,:)  ,ALLOCATABLE :: PIN, SSIN, PBCL, PFCL, PRF, SPE   ! (NRM,NSTM)
  REAL(8), DIMENSION(:,:)  ,ALLOCATABLE :: RGFLX                             ! (NRM,NSM)
  REAL(8), DIMENSION(:,:)  ,ALLOCATABLE :: AJRFV                             ! (NRM,3)
  REAL(8), DIMENSION(:,:,:),ALLOCATABLE :: PRFV                              ! (NRM,NSTM,3)

!     ****** COEFFICIENT VARIABLES ******
! TRCEF
  REAL(8), DIMENSION(:)  ,ALLOCATABLE :: ETA, S, ALPHA, RKCV, TAUB, TAUF, TAUK                                     ! (NRM)
  REAL(8), DIMENSION(:,:),ALLOCATABLE :: AK, AVK, AD, AV, AKNC, AKDW, ADNC, ADDW, AVNC, AVDW, AVKNC, AVKDW ! (NRM,NSTM)
  REAL(8), DIMENSION(:,:),ALLOCATABLE :: VGR1, VGR2, VGR3, VGR4                                            ! (NRM,4)
  CHARACTER(LEN=40)       :: KGR1, KGR2, KGR3, KGR4

!     ****** GLOBAL VARIABLES ******
! TRGLB
  REAL(8)  ::  WBULKT, WTAILT, WPT, AJT, AJOHT, AJNBT, AJRFT, AJBST, AJTTOR, PINT, POHT, PNBT, PNFT,      &
 &     PBINT, PFINT, POUT, PCXT, PIET, PRLT, PEXST, PRFST, SINT, SIET, SNBT, SNFT, SOUT, VLOOP, ALI, RQ1, &
 &     RPE, Q0, ZEFF0, QF, WPDOT, TAUE1, TAUE2, TAUE89, TAUE98, H98Y2, BETAP0, BETAPA, BETA0, BETAA, BETAQ0, BETAN
  REAL(8), DIMENSION(3)   :: AJRFVT
  REAL(8), DIMENSION(NFM) :: ANF0, TF0, ANFAV, TFAV, WFT
  REAL(8), DIMENSION(:)  ,ALLOCATABLE :: ANS0, TS0, ANSAV, ANLAV, TSAV, WST, PRFT, PBCLT, PFCLT, PLT, SPET, SLT  ! (NSTM)
  REAL(8), DIMENSION(:,:),ALLOCATABLE :: PRFVT                                ! (NSTM,3)

!     ****** GRAPHIC DATA VARIABLES ******
! TRPLR
  REAL(4), DIMENSION(NTM) :: GT, GTS
  REAL(4), DIMENSION(NGM) :: GTR
  REAL(4), DIMENSION(NLM) :: GBL
  REAL(4), DIMENSION(NTM,8) :: GYT
  REAL(4), DIMENSION(NTM,NCTM) :: GVT
  REAL(4), DIMENSION(NLM,10) :: GBR, GBRH, GBP1, GBAN
  REAL(4), DIMENSION(:)    ,ALLOCATABLE :: GRM                       ! (NRM)
  REAL(4), DIMENSION(:)    ,ALLOCATABLE :: GRG                       ! (NRMP)
  REAL(4), DIMENSION(:,:)  ,ALLOCATABLE :: GJB, GAD                  ! (NRMP,4)
  REAL(4), DIMENSION(:,:)  ,ALLOCATABLE :: GET                       ! (NRMP,5)
  REAL(4), DIMENSION(:,:)  ,ALLOCATABLE :: GAK                       ! (NRMP,6)
  REAL(4), DIMENSION(:,:)  ,ALLOCATABLE :: GYR, GER                  ! (NRMP,8)
  REAL(4), DIMENSION(:,:)  ,ALLOCATABLE :: GPNB                      ! (4*NRM,10)
  REAL(4), DIMENSION(:,:,:),ALLOCATABLE :: GVR                       ! (NRMP,NGM,NCGM)
  REAL(4), DIMENSION(:,:,:),ALLOCATABLE :: G3D                       ! (NRM,NGM,NCTM)
  INTEGER(4)              :: NGR, NGT, NGST
  INTEGER(4),DIMENSION(10):: NLMAX

!     ****** EQUILIBRIUM INTERFACE VARIABLES ******
! TREQV1
  REAL(8)                 :: BPSEQ,RIPEQ
  REAL(8), DIMENSION(:),ALLOCATABLE :: &
       & RHOTR, PRHO, HJRHO, VTRHO, TRHO, TTRHO, DVRHO, RKPRHO, ABRHO, &
       & ARRHO, AR1RHO, AR2RHO,  RJCB, EPSRHO, BPRHO, RMJRHO, RMNRHO, &
       & TTRHOG, DVRHOG, RKPRHOG, ABRHOG, ARRHOG, AR1RHOG, AR2RHOG, &
       & ABB2RHOG, AIB2RHOG, ARHBRHOG, RMJRHOG, RMNRHOG, &
       & PSITRHO, PSIPRHO, PPPRHO, PIQRHO, PIRHO, FACTQ   ! (NRM)
  REAL(8), DIMENSION(:),ALLOCATABLE :: QRHO  ! (NRMP)

!     ****** LOG FILE NAME ******
! TRNAM
  CHARACTER(LEN=80) :: KNAMEQ,KNAMTR,KFNLOG

!     ****** ADDITIONAL PART BY HONDA MITSURU ******
! TRADD
  REAL(8), PARAMETER          :: VOID = 0.D0
  REAL(8), DIMENSION(3,3)     :: FA, FB, FC
  REAL(8), DIMENSION(:)      ,ALLOCATABLE :: RTM, AMZ, PEXT, PNSSA, PNSA, PTSA   ! (NSTM)
  REAL(8), DIMENSION(:,:)    ,ALLOCATABLE :: PEX, SEX                            ! (NRM,NSTM)
  REAL(8), DIMENSION(:,:,:)  ,ALLOCATABLE :: AKDWD, AKDWP, ADDWD, ADDWP          ! (NRM,NSM,NSM)
  REAL(8), DIMENSION(:,:,:,:),ALLOCATABLE :: VV,DD                               ! (NVM,NVM,4,3)
  REAL(8), DIMENSION(:,:,:,:),ALLOCATABLE :: VI,DI                               ! (NVM,NVM,2,3)
  INTEGER(4)                      :: NTEQIT

!     ****** MODEL SELECTION VARIABLES ******
! TRMDS
  REAL(8)     :: SUMPBM
  INTEGER(4)  :: NEQMAX, MDLEQB, MDLEQN, MDLEQT, MDLEQU, MDLEQZ, MDLEQ0, MDLEQE, MDLEOI, &
 &     NSCMAX, NSTMAX, MDLWLD, MDDIAG, MDDW, MDLFLX, MDLER, MDCD05, MDEDGE
  INTEGER(4), DIMENSION(:)  ,ALLOCATABLE :: NSS, NSV, NNS, NST                   ! (NEQM)
  INTEGER(4), DIMENSION(:,:),ALLOCATABLE :: NEA                                  ! (0:NSTM,0:3)

!     ****** NCLASS ******
! TRNCL
  REAL(8), DIMENSION(:)    ,ALLOCATABLE :: AJBSNC, ETANC, AJEXNC                               !  (NRM)
  REAL(8), DIMENSION(:,:)  ,ALLOCATABLE :: CJBSP, CJBST, ADNCG, AVNCG                          !  (NRM,NSM)
  REAL(8), DIMENSION(:,:,:),ALLOCATABLE :: AKNCP, AKNCT, ADNCP, ADNCT, AKLP, AKLD, ADLP, ADLD  !  (NRM,NSM,NSM)
  REAL(8), DIMENSION(:,:,:),ALLOCATABLE :: RGFLS,   RQFLS                                      !  (NRM,5,NSM)
  INTEGER(4) :: MDNCLS, NSLMAX

!     ****** STORED VARIABLES FOR UFILE ******
! TRSVU
  REAL(8), DIMENSION(NTUM) :: PNFU, PNFUA, RRU, RAU, PHIAU, VOLAU, RIPU, BBU, RKAPU, PNBIU   !
  REAL(8), DIMENSION(:,:)  ,ALLOCATABLE :: QPU,  AJU, AJNBU, BPU, PRLU, PECU, POHU, DVRHOU, AJBSU, ZEFFU, PBMU, &
 &     RNFU, WROTU, ZEFFU_ORG, RKPRHOU, RMJRHOU, RMNRHOU, ARRHOU, AR1RHOU, AR2RHOU, ABRHOU, TTRHOU, SWLU  ! (NTUM,NRMP)
  REAL(8), DIMENSION(:,:)  ,ALLOCATABLE :: PTSU, PNSU, PTSUA, PNSUA                                       ! (NTUM,NSTM)
  REAL(8), DIMENSION(:,:,:),ALLOCATABLE :: RNU, RTU, PNBU, PICU, SNBU,RNU_ORG                             ! (NTUM,NRMP,NSM)

!     ****** UFILE CONTROL ******
! TRUFL
  REAL(8)    :: TIME_INT
  INTEGER(4) :: MDLXP,MDLUF, MODEP,MDNI,MDCURT,MDNM1,MDLJQ,MDPHIA,NTS
  CHARACTER(LEN=80) :: KUFDIR,KUFDEV,KUFDCG
!     ****** LAPACK ******
! TRLPCK
  INTEGER(4) :: MDLPCK

!     ************
!TRPROF
  REAL(8), DIMENSION(:),ALLOCATABLE ::  PNSSO,PTSO,PNSSAO,PTSAO                              ! (NSTM)

!     ******************************************************
  CONTAINS

  SUBROUTINE ALLOCATE_TRCOMM(ierr)

    use trcom1, ONLY : ALLOCATE_TRCOM1
    integer(4), intent(out):: ierr

    ierr = 0
    if(nsmax<1) then
      write(6,*) "XXX ALLOCATE_TRCOMM : ILLEGAL PARAMETER    NSMAX=",nsmax
      ierr = 1
      return
    endif
    if(nrmax<1) then
      write(6,*) "XXX ALLOCATE_TRCOMM : ILLEGAL PARAMETER    NRMAX=",nrmax
      ierr = 1
      return
    endif

    if(nrmax_old==nrmax .and. nsmax_old==nsmax .and. nszmax_old==nszmax .and. nsnmax_old==nsnmax .and. allocated(PNSS)) return
    if(allocated(PNSS)) call DEALLOCATE_TRCOMM

    NSTMAX  = NSMAX+NSZMAX+NSNMAX
    NEQMAXM = 3*NSTMAX+1
    NVM     = NEQMAXM
    MWM     = 4*NEQMAXM-1
    MLM     = NEQMAXM*NRMAX
    NRMP    = NRMAX+1
    NGLF    = NRMAX
    LDAB    = 6*NEQMAXM


!    ALLOCATE(PA(NSTMAX),PZ(NSTMAX),PN(NSTMAX),PNS(NSTMAX),PT(NSTMAX),PTS(NSTMAX),PELPAT(NSTMAX),STAT=IERR)
!      IF(IERR.NE.0) GOTO 900

!    ALLOCATE(PNSS(NSTMAX) ,STAT=IERR)
    ALLOCATE(PNSS(NSTM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900

    ALLOCATE(XV(NEQMAXM,NRMAX),STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(YV(NFM,NRMAX), AY(NFM,NRMAX), Y(NFM,NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(ZV(NSMAX,NRMAX), AZ(NSMAX,NRMAX), Z(NSMAX,NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(AX(6*NEQMAXM,NEQMAXM*NRMAX),X(NEQMAXM*NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900

    ALLOCATE(RG(NRMAX),RM(NRM),RHOM(NRMAX),RHOG(NRMAX),BP(NRMAX),RDP(NRMAX),RPSI(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(RN(NRM,NSTM),RT(NRM,NSTM),RU(NRMAX,NSTM),RW(NRMAX,NFM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900

    ALLOCATE(RNF(NRMAX,NFM),RTF(NRMAX,NFM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(ANC(NRMAX),ANFE(NRMAX),ANNU(NRMAX),ZEFF(NRMAX),PZC(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(PZFE(NRMAX),BETA(NRMAX),BETAP(NRMAX),BETAL(NRMAX),BETAPL(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(BETAQ(NRMAX),PBM(NRMAX),PADD(NRMAX),VTOR(NRMAX),VPAR(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(VPRP(NRMAX),VPOL(NRMAX),WROT(NRMAX),ER(NRMAX),VEXB(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(WEXB(NRMAX),AGMP(NRMAX),AJ(NRMAX),AJOH(NRMAX), EZOH(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(QP(NRMAX),AJTOR(NRMAX),AJNB(NRMAX),AJRF(NRMAX),AJBS(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(PNB(NRMAX),SNB(NRMAX),PBIN(NRMAX),PNF(NRMAX),SNF(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(PFIN(NRMAX),POH(NRMAX),PRL(NRMAX),PCX(NRMAX),PIE(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(SIE(NRMAX),SCX(NRMAX),TSIE(NRMAX),TSCX(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(PIN(NRMAX,NSTM),SSIN(NRMAX,NSTM),PBCL(NRMAX,NSTM), SPE(NRMAX,NSTM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(PFCL(NRMAX,NSTM),PRF(NRMAX,NSTM), STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(PRFV(NRMAX,NSTM,3),AJRFV(NRMAX,3),RGFLX(NRMAX,NSM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900

    ALLOCATE(ETA(NRMAX),S(NRMAX),ALPHA(NRMAX),RKCV(NRMAX),TAUB(NRMAX),TAUF(NRMAX),TAUK(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(AK(NRMAX,NSTM),AVK(NRMAX,NSTM),AD(NRMAX,NSTM),AV(NRMAX,NSTM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(AKNC(NRMAX,NSTM),AKDW(NRMAX,NSTM),ADNC(NRMAX,NSTM),ADDW(NRMAX,NSTM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(AVNC(NRMAX,NSTM),AVDW(NRMAX,NSTM),AVKNC(NRMAX,NSTM),AVKDW(NRMAX,NSTM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(VGR1(NRMAX,4),VGR2(NRMAX,4),VGR3(NRMAX,4),VGR4(NRMAX,4) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900

    ALLOCATE(ANS0(NSTM),TS0(NSTM),ANSAV(NSTM),ANLAV(NSTM),TSAV(NSTM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(WST(NSTM),PRFT(NSTM),PBCLT(NSTM),PFCLT(NSTM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(PLT(NSTM),SPET(NSTM),SLT(NSTM),PRFVT(NSTM,3) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900

    ALLOCATE(GRM(NRMAX),GRG(NRMP),GJB(NRMP,4),GAD(NRMP,4) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(GET(NRMP,5),GAK(NRMP,6),GYR(NRMP,8),GER(NRMP,8) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(GPNB(4*NRMAX,10),GVR(NRMP,NGM,NCGM),G3D(NRMAX,NGM,NCTM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900

    ALLOCATE(RHOTR(NRMAX),PRHO(NRMAX),HJRHO(NRMAX),VTRHO(NRMAX),TRHO(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(TTRHO(NRMAX),DVRHO(NRMAX),RKPRHO(NRMAX),ABRHO(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(PSITRHO(NRMAX),PSIPRHO(NRMAX),PPPRHO(NRMAX),PIQRHO(NRMAX),PIRHO(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(ARRHO(NRMAX),AR1RHO(NRMAX),AR2RHO(NRMAX),RJCB(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(EPSRHO(NRMAX),BPRHO(NRMAX),RMJRHO(NRMAX),RMNRHO(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(TTRHOG(NRMAX),DVRHOG(NRMAX),RKPRHOG(NRMAX),ABRHOG(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(ARRHOG(NRMAX),AR1RHOG(NRMAX),AR2RHOG(NRMAX),ABB2RHOG(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(AIB2RHOG(NRMAX),ARHBRHOG(NRMAX),RMJRHOG(NRMAX),RMNRHOG(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(FACTQ(NRMAX),QRHO(NRMP) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900

    ALLOCATE(RTM(NSTM),AMZ(NSTM),PEXT(NSTM),PNSSA(NSTM),PNSA(NSTM),PTSA(NSTM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(PEX(NRMAX,NSTM), SEX(NRMAX,NSTM),AKDWD(NRMAX,NSMAX,NSMAX)  ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(AKDWP(NRMAX,NSMAX,NSMAX),ADDWD(NRMAX,NSMAX,NSMAX),ADDWP(NRMAX,NSMAX,NSMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(VV(NEQMAXM,NEQMAXM,4,3),DD(NEQMAXM,NEQMAXM,4,3) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(VI(NEQMAXM,NEQMAXM,2,3),DI(NEQMAXM,NEQMAXM,2,3) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900

    ALLOCATE(NSS(NEQMAXM),NSV(NEQMAXM),NNS(NEQMAXM),NST(NEQMAXM),NEA(0:NSTM,0:3) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900

    ALLOCATE(AJBSNC(NRMAX), ETANC(NRMAX), AJEXNC(NRMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(CJBSP(NRMAX,NSM),CJBST(NRMAX,NSM),ADNCG(NRMAX,NSM),AVNCG(NRMAX,NSM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(AKNCP(NRMAX,NSM,NSM),AKNCT(NRMAX,NSM,NSM),ADNCP(NRMAX,NSM,NSM),ADNCT(NRMAX,NSM,NSM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(AKLP(NRMAX,NSM,NSM),AKLD(NRMAX,NSM,NSM),ADLP(NRMAX,NSM,NSM),ADLD(NRMAX,NSM,NSM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(RGFLS(NRMAX,5,NSM),RQFLS(NRMAX,5,NSM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900

    ALLOCATE(QPU(NTUM,NRMP),AJU(NTUM,NRMP),AJNBU(NTUM,NRMP),BPU(NTUM,NRMP) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(PRLU(NTUM,NRMP),PECU(NTUM,NRMP),POHU(NTUM,NRMP),DVRHOU(NTUM,NRMP) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(AJBSU(NTUM,NRMP),ZEFFU(NTUM,NRMP),PBMU(NTUM,NRMP),RNFU(NTUM,NRMP) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(WROTU(NTUM,NRMP),ZEFFU_ORG(NTUM,NRMP),RKPRHOU(NTUM,NRMP) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(RMJRHOU(NTUM,NRMP),RMNRHOU(NTUM,NRMP),ARRHOU(NTUM,NRMP) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(AR1RHOU(NTUM,NRMP),AR2RHOU(NTUM,NRMP),ABRHOU(NTUM,NRMP) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(TTRHOU(NTUM,NRMP),SWLU(NTUM,NRMP) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(PTSU(NTUM,NSTM),PNSU(NTUM,NSTM),PTSUA(NTUM,NSTM),PNSUA(NTUM,NSTM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(RNU(NTUM,NRMP,NSM),RTU(NTUM,NRMP,NSM),PNBU(NTUM,NRMP,NSMAX) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900
    ALLOCATE(PICU(NTUM,NRMP,NSMAX),SNBU(NTUM,NRMP,NSMAX),RNU_ORG(NTUM,NRMP,NSM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900

    ALLOCATE(PNSSO(NSTM),PTSO(NSTM),PNSSAO(NSTM),PTSAO(NSTM) ,STAT=IERR)
      IF(IERR.NE.0) GOTO 900


    CALL ALLOCATE_TRCOM1(IERR)
      IF(IERR.NE.0) GOTO 900

    nrmax_old  = nrmax
    nsmax_old  = nsmax
    nszmax_old = nszmax
    nsnmax_old = nsnmax
    return

 900 continue
    write(6,*) "XX  TRCOMM ALLOCATION ERROR IERR=",ierr
    call DEALLOCATE_ERR_TRCOMM
    return

  END SUBROUTINE ALLOCATE_TRCOMM


  SUBROUTINE DEALLOCATE_TRCOMM
    use trcom1, ONLY : DEALLOCATE_TRCOM1

!    DEALLOCATE(PA,PZ,PN,PNS,PT,PTS,PELPAT)
    DEALLOCATE(PNSS)
    DEALLOCATE(XV,YV, AY, Y,ZV, AZ, Z,AX,X)
    DEALLOCATE(RG,RM,RHOM,RHOG,BP,RDP,RPSI,RN,RT,RU,RW)
    DEALLOCATE(RNF,RTF,ANC,ANFE,ANNU,ZEFF,PZC,PZFE,BETA,BETAP,BETAL,BETAPL)
    DEALLOCATE(BETAQ,PBM,PADD,VTOR,VPAR,VPRP,VPOL,WROT,ER,VEXB,WEXB,AGMP)
    DEALLOCATE(AJ,AJOH, EZOH,QP,AJTOR,AJNB,AJRF,AJBS)
    DEALLOCATE(PNB,SNB,PBIN,PNF,SNF,PFIN,POH,PRL,PCX,PIE)
    DEALLOCATE(SIE,SCX,TSIE,TSCX,PIN,SSIN,PBCL, SPE)
    DEALLOCATE(PFCL,PRF,PRFV,AJRFV,RGFLX)
    DEALLOCATE(ETA,S,ALPHA,RKCV,TAUB,TAUF,TAUK,AK,AVK,AD,AV)
    DEALLOCATE(AKNC,AKDW,ADNC,ADDW,AVNC,AVDW,AVKNC,AVKDW)
    DEALLOCATE(VGR1,VGR2,VGR3,VGR4)
    DEALLOCATE(ANS0,TS0,ANSAV,ANLAV,TSAV,WST,PRFT,PBCLT,PFCLT)
    DEALLOCATE(PLT,SPET,SLT,PRFVT)
    DEALLOCATE(GRM,GRG,GJB,GAD,GET,GAK,GYR,GER,GPNB,GVR,G3D)
    DEALLOCATE(RHOTR,PRHO,HJRHO,VTRHO,TRHO,TTRHO,DVRHO,RKPRHO,ABRHO)
    DEALLOCATE(PSITRHO,PSIPRHO,PPPRHO,PIQRHO, PIRHO)
    DEALLOCATE(ARRHO,AR1RHO,AR2RHO,RJCB,EPSRHO,BPRHO,RMJRHO,RMNRHO)
    DEALLOCATE(TTRHOG,DVRHOG,RKPRHOG,ABRHOG,ARRHOG,AR1RHOG,AR2RHOG,ABB2RHOG)
    DEALLOCATE(AIB2RHOG,ARHBRHOG,RMJRHOG,RMNRHOG,FACTQ,QRHO)
    DEALLOCATE(RTM,AMZ,PEXT,PNSSA,PNSA,PTSA,PEX, SEX,AKDWD)
    DEALLOCATE(AKDWP,ADDWD,ADDWP,VV,DD,VI,DI)
    DEALLOCATE(NSS,NSV,NNS,NST,NEA)
    DEALLOCATE(AJBSNC, ETANC, AJEXNC,CJBSP,CJBST,ADNCG,AVNCG)
    DEALLOCATE(AKNCP,AKNCT,ADNCP,ADNCT,AKLP,AKLD,ADLP,ADLD,RGFLS,RQFLS)
    DEALLOCATE(QPU,AJU,AJNBU,BPU,PRLU,PECU,POHU,DVRHOU)
    DEALLOCATE(AJBSU,ZEFFU,PBMU,RNFU,WROTU,ZEFFU_ORG,RKPRHOU)
    DEALLOCATE(RMJRHOU,RMNRHOU,ARRHOU,AR1RHOU,AR2RHOU,ABRHOU)
    DEALLOCATE(TTRHOU,SWLU,PTSU,PNSU,PTSUA,PNSUA)
    DEALLOCATE(RNU,RTU,PNBU,PICU,SNBU,RNU_ORG)
    DEALLOCATE(PNSSO,PTSO,PNSSAO,PTSAO)

    CALL  DEALLOCATE_TRCOM1

    return

  END SUBROUTINE DEALLOCATE_TRCOMM


  SUBROUTINE DEALLOCATE_ERR_TRCOMM
    use trcom1, ONLY : DEALLOCATE_ERR_TRCOM1

!    IF(ALLOCATED(PA       ))     DEALLOCATE(PA       )
!    IF(ALLOCATED(PZ       ))     DEALLOCATE(PZ       )
!    IF(ALLOCATED(PN       ))     DEALLOCATE(PN       )
!    IF(ALLOCATED(PNS      ))     DEALLOCATE(PNS      )
!    IF(ALLOCATED(PT       ))     DEALLOCATE(PT       )
!    IF(ALLOCATED(PTS      ))     DEALLOCATE(PTS      )
!    IF(ALLOCATED(PELPAT   ))     DEALLOCATE(PELPAT   )
    IF(ALLOCATED(PNSS     ))     DEALLOCATE(PNSS     )
    IF(ALLOCATED(XV       ))     DEALLOCATE(XV       )
    IF(ALLOCATED(YV       ))     DEALLOCATE(YV       )
    IF(ALLOCATED(AY       ))     DEALLOCATE(AY       )
    IF(ALLOCATED(Y        ))     DEALLOCATE(Y        )
    IF(ALLOCATED(ZV       ))     DEALLOCATE(ZV       )
    IF(ALLOCATED(AZ       ))     DEALLOCATE(AZ       )
    IF(ALLOCATED(Z        ))     DEALLOCATE(Z        )
    IF(ALLOCATED(AX       ))     DEALLOCATE(AX       )
    IF(ALLOCATED(X        ))     DEALLOCATE(X        )
    IF(ALLOCATED(RG       ))     DEALLOCATE(RG       )
    IF(ALLOCATED(RM       ))     DEALLOCATE(RM       )
    IF(ALLOCATED(RHOM     ))     DEALLOCATE(RHOM     )
    IF(ALLOCATED(RHOG     ))     DEALLOCATE(RHOG     )
    IF(ALLOCATED(BP       ))     DEALLOCATE(BP       )
    IF(ALLOCATED(RDP      ))     DEALLOCATE(RDP      )
    IF(ALLOCATED(RPSI     ))     DEALLOCATE(RPSI     )
    IF(ALLOCATED(RN       ))     DEALLOCATE(RN       )
    IF(ALLOCATED(RT       ))     DEALLOCATE(RT       )
    IF(ALLOCATED(RU       ))     DEALLOCATE(RU       )
    IF(ALLOCATED(RW       ))     DEALLOCATE(RW       )
    IF(ALLOCATED(RNF      ))     DEALLOCATE(RNF      )
    IF(ALLOCATED(RTF      ))     DEALLOCATE(RTF      )
    IF(ALLOCATED(ANC      ))     DEALLOCATE(ANC      )
    IF(ALLOCATED(ANFE     ))     DEALLOCATE(ANFE     )
    IF(ALLOCATED(ANNU     ))     DEALLOCATE(ANNU     )
    IF(ALLOCATED(ZEFF     ))     DEALLOCATE(ZEFF     )
    IF(ALLOCATED(PZC      ))     DEALLOCATE(PZC      )
    IF(ALLOCATED(PZFE     ))     DEALLOCATE(PZFE     )
    IF(ALLOCATED(BETA     ))     DEALLOCATE(BETA     )
    IF(ALLOCATED(BETAP    ))     DEALLOCATE(BETAP    )
    IF(ALLOCATED(BETAL    ))     DEALLOCATE(BETAL    )
    IF(ALLOCATED(BETAPL   ))     DEALLOCATE(BETAPL   )
    IF(ALLOCATED(BETAQ    ))     DEALLOCATE(BETAQ    )
    IF(ALLOCATED(PBM      ))     DEALLOCATE(PBM      )
    IF(ALLOCATED(PADD     ))     DEALLOCATE(PADD     )
    IF(ALLOCATED(VTOR     ))     DEALLOCATE(VTOR     )
    IF(ALLOCATED(VPAR     ))     DEALLOCATE(VPAR     )
    IF(ALLOCATED(VPRP     ))     DEALLOCATE(VPRP     )
    IF(ALLOCATED(VPOL     ))     DEALLOCATE(VPOL     )
    IF(ALLOCATED(WROT     ))     DEALLOCATE(WROT     )
    IF(ALLOCATED(ER       ))     DEALLOCATE(ER       )
    IF(ALLOCATED(VEXB     ))     DEALLOCATE(VEXB     )
    IF(ALLOCATED(WEXB     ))     DEALLOCATE(WEXB     )
    IF(ALLOCATED(AGMP     ))     DEALLOCATE(AGMP     )
    IF(ALLOCATED(AJ       ))     DEALLOCATE(AJ       )
    IF(ALLOCATED(AJOH     ))     DEALLOCATE(AJOH     )
    IF(ALLOCATED(EZOH     ))     DEALLOCATE(EZOH     )
    IF(ALLOCATED(QP       ))     DEALLOCATE(QP       )
    IF(ALLOCATED(AJTOR    ))     DEALLOCATE(AJTOR    )
    IF(ALLOCATED(AJNB     ))     DEALLOCATE(AJNB     )
    IF(ALLOCATED(AJRF     ))     DEALLOCATE(AJRF     )
    IF(ALLOCATED(AJBS     ))     DEALLOCATE(AJBS     )
    IF(ALLOCATED(PNB      ))     DEALLOCATE(PNB      )
    IF(ALLOCATED(SNB      ))     DEALLOCATE(SNB      )
    IF(ALLOCATED(PBIN     ))     DEALLOCATE(PBIN     )
    IF(ALLOCATED(PNF      ))     DEALLOCATE(PNF      )
    IF(ALLOCATED(SNF      ))     DEALLOCATE(SNF      )
    IF(ALLOCATED(PFIN     ))     DEALLOCATE(PFIN     )
    IF(ALLOCATED(POH      ))     DEALLOCATE(POH      )
    IF(ALLOCATED(PRL      ))     DEALLOCATE(PRL      )
    IF(ALLOCATED(PCX      ))     DEALLOCATE(PCX      )
    IF(ALLOCATED(PIE      ))     DEALLOCATE(PIE      )
    IF(ALLOCATED(SIE      ))     DEALLOCATE(SIE      )
    IF(ALLOCATED(SCX      ))     DEALLOCATE(SCX      )
    IF(ALLOCATED(TSIE     ))     DEALLOCATE(TSIE     )
    IF(ALLOCATED(TSCX     ))     DEALLOCATE(TSCX     )
    IF(ALLOCATED(PIN      ))     DEALLOCATE(PIN      )
    IF(ALLOCATED(SSIN     ))     DEALLOCATE(SSIN     )
    IF(ALLOCATED(PBCL     ))     DEALLOCATE(PBCL     )
    IF(ALLOCATED(SPE      ))     DEALLOCATE(SPE      )
    IF(ALLOCATED(PFCL     ))     DEALLOCATE(PFCL     )
    IF(ALLOCATED(PRF      ))     DEALLOCATE(PRF      )
    IF(ALLOCATED(PRFV     ))     DEALLOCATE(PRFV     )
    IF(ALLOCATED(AJRFV    ))     DEALLOCATE(AJRFV    )
    IF(ALLOCATED(RGFLX    ))     DEALLOCATE(RGFLX    )
    IF(ALLOCATED(ETA      ))     DEALLOCATE(ETA      )
    IF(ALLOCATED(S        ))     DEALLOCATE(S        )
    IF(ALLOCATED(ALPHA    ))     DEALLOCATE(ALPHA    )
    IF(ALLOCATED(RKCV     ))     DEALLOCATE(RKCV     )
    IF(ALLOCATED(TAUB     ))     DEALLOCATE(TAUB     )
    IF(ALLOCATED(TAUF     ))     DEALLOCATE(TAUF     )
    IF(ALLOCATED(TAUK     ))     DEALLOCATE(TAUK     )
    IF(ALLOCATED(AK       ))     DEALLOCATE(AK       )
    IF(ALLOCATED(AVK      ))     DEALLOCATE(AVK      )
    IF(ALLOCATED(AD       ))     DEALLOCATE(AD       )
    IF(ALLOCATED(AV       ))     DEALLOCATE(AV       )
    IF(ALLOCATED(AKNC     ))     DEALLOCATE(AKNC     )
    IF(ALLOCATED(AKDW     ))     DEALLOCATE(AKDW     )
    IF(ALLOCATED(ADNC     ))     DEALLOCATE(ADNC     )
    IF(ALLOCATED(ADDW     ))     DEALLOCATE(ADDW     )
    IF(ALLOCATED(AVNC     ))     DEALLOCATE(AVNC     )
    IF(ALLOCATED(AVDW     ))     DEALLOCATE(AVDW     )
    IF(ALLOCATED(AVKNC    ))     DEALLOCATE(AVKNC    )
    IF(ALLOCATED(AVKDW    ))     DEALLOCATE(AVKDW    )
    IF(ALLOCATED(VGR1     ))     DEALLOCATE(VGR1     )
    IF(ALLOCATED(VGR2     ))     DEALLOCATE(VGR2     )
    IF(ALLOCATED(VGR3     ))     DEALLOCATE(VGR3     )
    IF(ALLOCATED(VGR4     ))     DEALLOCATE(VGR4     )
    IF(ALLOCATED(ANS0     ))     DEALLOCATE(ANS0     )
    IF(ALLOCATED(TS0      ))     DEALLOCATE(TS0      )
    IF(ALLOCATED(ANSAV    ))     DEALLOCATE(ANSAV    )
    IF(ALLOCATED(ANLAV    ))     DEALLOCATE(ANLAV    )
    IF(ALLOCATED(TSAV     ))     DEALLOCATE(TSAV     )
    IF(ALLOCATED(WST      ))     DEALLOCATE(WST      )
    IF(ALLOCATED(PRFT     ))     DEALLOCATE(PRFT     )
    IF(ALLOCATED(PBCLT    ))     DEALLOCATE(PBCLT    )
    IF(ALLOCATED(PFCLT    ))     DEALLOCATE(PFCLT    )
    IF(ALLOCATED(PLT      ))     DEALLOCATE(PLT      )
    IF(ALLOCATED(SPET     ))     DEALLOCATE(SPET     )
    IF(ALLOCATED(SLT      ))     DEALLOCATE(SLT      )
    IF(ALLOCATED(PRFVT    ))     DEALLOCATE(PRFVT    )
    IF(ALLOCATED(GRM      ))     DEALLOCATE(GRM      )
    IF(ALLOCATED(GRG      ))     DEALLOCATE(GRG      )
    IF(ALLOCATED(GJB      ))     DEALLOCATE(GJB      )
    IF(ALLOCATED(GAD      ))     DEALLOCATE(GAD      )
    IF(ALLOCATED(GET      ))     DEALLOCATE(GET      )
    IF(ALLOCATED(GAK      ))     DEALLOCATE(GAK      )
    IF(ALLOCATED(GYR      ))     DEALLOCATE(GYR      )
    IF(ALLOCATED(GER      ))     DEALLOCATE(GER      )
    IF(ALLOCATED(GPNB     ))     DEALLOCATE(GPNB     )
    IF(ALLOCATED(GVR      ))     DEALLOCATE(GVR      )
    IF(ALLOCATED(G3D      ))     DEALLOCATE(G3D      )
    IF(ALLOCATED(RHOTR    ))     DEALLOCATE(RHOTR    )
    IF(ALLOCATED(PRHO     ))     DEALLOCATE(PRHO     )
    IF(ALLOCATED(HJRHO    ))     DEALLOCATE(HJRHO    )
    IF(ALLOCATED(VTRHO    ))     DEALLOCATE(VTRHO    )
    IF(ALLOCATED(TRHO     ))     DEALLOCATE(TRHO     )
    IF(ALLOCATED(TTRHO    ))     DEALLOCATE(TTRHO    )
    IF(ALLOCATED(PSITRHO    ))     DEALLOCATE(PSITRHO    )
    IF(ALLOCATED(PSIPRHO    ))     DEALLOCATE(PSIPRHO    )
    IF(ALLOCATED(PPPRHO    ))     DEALLOCATE(PPPRHO    )
    IF(ALLOCATED(PIQRHO    ))     DEALLOCATE(PIQRHO    )
    IF(ALLOCATED(PIRHO    ))     DEALLOCATE(PIRHO    )
    IF(ALLOCATED(DVRHO    ))     DEALLOCATE(DVRHO    )
    IF(ALLOCATED(RKPRHO   ))     DEALLOCATE(RKPRHO   )
    IF(ALLOCATED(ABRHO    ))     DEALLOCATE(ABRHO    )
    IF(ALLOCATED(ARRHO    ))     DEALLOCATE(ARRHO    )
    IF(ALLOCATED(AR1RHO   ))     DEALLOCATE(AR1RHO   )
    IF(ALLOCATED(AR2RHO   ))     DEALLOCATE(AR2RHO   )
    IF(ALLOCATED(RJCB     ))     DEALLOCATE(RJCB     )
    IF(ALLOCATED(EPSRHO   ))     DEALLOCATE(EPSRHO   )
    IF(ALLOCATED(BPRHO    ))     DEALLOCATE(BPRHO    )
    IF(ALLOCATED(RMJRHO   ))     DEALLOCATE(RMJRHO   )
    IF(ALLOCATED(RMNRHO   ))     DEALLOCATE(RMNRHO   )
    IF(ALLOCATED(TTRHOG   ))     DEALLOCATE(TTRHOG   )
    IF(ALLOCATED(DVRHOG   ))     DEALLOCATE(DVRHOG   )
    IF(ALLOCATED(RKPRHOG  ))     DEALLOCATE(RKPRHOG  )
    IF(ALLOCATED(ABRHOG   ))     DEALLOCATE(ABRHOG   )
    IF(ALLOCATED(ARRHOG   ))     DEALLOCATE(ARRHOG   )
    IF(ALLOCATED(AR1RHOG  ))     DEALLOCATE(AR1RHOG  )
    IF(ALLOCATED(AR2RHOG  ))     DEALLOCATE(AR2RHOG  )
    IF(ALLOCATED(ABB2RHOG ))     DEALLOCATE(ABB2RHOG )
    IF(ALLOCATED(AIB2RHOG ))     DEALLOCATE(AIB2RHOG )
    IF(ALLOCATED(ARHBRHOG ))     DEALLOCATE(ARHBRHOG )
    IF(ALLOCATED(RMJRHOG  ))     DEALLOCATE(RMJRHOG  )
    IF(ALLOCATED(RMNRHOG  ))     DEALLOCATE(RMNRHOG  )
    IF(ALLOCATED(FACTQ    ))     DEALLOCATE(FACTQ    )
    IF(ALLOCATED(QRHO     ))     DEALLOCATE(QRHO     )
    IF(ALLOCATED(RTM      ))     DEALLOCATE(RTM      )
    IF(ALLOCATED(AMZ      ))     DEALLOCATE(AMZ      )
    IF(ALLOCATED(PEXT     ))     DEALLOCATE(PEXT     )
    IF(ALLOCATED(PNSSA    ))     DEALLOCATE(PNSSA    )
    IF(ALLOCATED(PNSA     ))     DEALLOCATE(PNSA     )
    IF(ALLOCATED(PTSA     ))     DEALLOCATE(PTSA     )
    IF(ALLOCATED(PEX      ))     DEALLOCATE(PEX      )
    IF(ALLOCATED(SEX      ))     DEALLOCATE(SEX      )
    IF(ALLOCATED(AKDWD    ))     DEALLOCATE(AKDWD    )
    IF(ALLOCATED(AKDWP    ))     DEALLOCATE(AKDWP    )
    IF(ALLOCATED(ADDWD    ))     DEALLOCATE(ADDWD    )
    IF(ALLOCATED(ADDWP    ))     DEALLOCATE(ADDWP    )
    IF(ALLOCATED(VV       ))     DEALLOCATE(VV       )
    IF(ALLOCATED(DD       ))     DEALLOCATE(DD       )
    IF(ALLOCATED(VI       ))     DEALLOCATE(VI       )
    IF(ALLOCATED(DI       ))     DEALLOCATE(DI       )
    IF(ALLOCATED(NSS      ))     DEALLOCATE(NSS      )
    IF(ALLOCATED(NSV      ))     DEALLOCATE(NSV      )
    IF(ALLOCATED(NNS      ))     DEALLOCATE(NNS      )
    IF(ALLOCATED(NST      ))     DEALLOCATE(NST      )
    IF(ALLOCATED(NEA      ))     DEALLOCATE(NEA      )
    IF(ALLOCATED(AJBSNC   ))     DEALLOCATE(AJBSNC   )
    IF(ALLOCATED(ETANC    ))     DEALLOCATE(ETANC    )
    IF(ALLOCATED(AJEXNC   ))     DEALLOCATE(AJEXNC   )
    IF(ALLOCATED(CJBSP    ))     DEALLOCATE(CJBSP    )
    IF(ALLOCATED(CJBST    ))     DEALLOCATE(CJBST    )
    IF(ALLOCATED(ADNCG    ))     DEALLOCATE(ADNCG    )
    IF(ALLOCATED(AVNCG    ))     DEALLOCATE(AVNCG    )
    IF(ALLOCATED(AKNCP    ))     DEALLOCATE(AKNCP    )
    IF(ALLOCATED(AKNCT    ))     DEALLOCATE(AKNCT    )
    IF(ALLOCATED(ADNCP    ))     DEALLOCATE(ADNCP    )
    IF(ALLOCATED(ADNCT    ))     DEALLOCATE(ADNCT    )
    IF(ALLOCATED(AKLP     ))     DEALLOCATE(AKLP     )
    IF(ALLOCATED(AKLD     ))     DEALLOCATE(AKLD     )
    IF(ALLOCATED(ADLP     ))     DEALLOCATE(ADLP     )
    IF(ALLOCATED(ADLD     ))     DEALLOCATE(ADLD     )
    IF(ALLOCATED(RGFLS    ))     DEALLOCATE(RGFLS    )
    IF(ALLOCATED(RQFLS    ))     DEALLOCATE(RQFLS    )
    IF(ALLOCATED(QPU      ))     DEALLOCATE(QPU      )
    IF(ALLOCATED(AJU      ))     DEALLOCATE(AJU      )
    IF(ALLOCATED(AJNBU    ))     DEALLOCATE(AJNBU    )
    IF(ALLOCATED(BPU      ))     DEALLOCATE(BPU      )
    IF(ALLOCATED(PRLU     ))     DEALLOCATE(PRLU     )
    IF(ALLOCATED(PECU     ))     DEALLOCATE(PECU     )
    IF(ALLOCATED(POHU     ))     DEALLOCATE(POHU     )
    IF(ALLOCATED(DVRHOU   ))     DEALLOCATE(DVRHOU   )
    IF(ALLOCATED(AJBSU    ))     DEALLOCATE(AJBSU    )
    IF(ALLOCATED(ZEFFU    ))     DEALLOCATE(ZEFFU    )
    IF(ALLOCATED(PBMU     ))     DEALLOCATE(PBMU     )
    IF(ALLOCATED(RNFU     ))     DEALLOCATE(RNFU     )
    IF(ALLOCATED(WROTU    ))     DEALLOCATE(WROTU    )
    IF(ALLOCATED(ZEFFU_ORG))     DEALLOCATE(ZEFFU_ORG)
    IF(ALLOCATED(RKPRHOU  ))     DEALLOCATE(RKPRHOU  )
    IF(ALLOCATED(RMJRHOU  ))     DEALLOCATE(RMJRHOU  )
    IF(ALLOCATED(RMNRHOU  ))     DEALLOCATE(RMNRHOU  )
    IF(ALLOCATED(ARRHOU   ))     DEALLOCATE(ARRHOU   )
    IF(ALLOCATED(AR1RHOU  ))     DEALLOCATE(AR1RHOU  )
    IF(ALLOCATED(AR2RHOU  ))     DEALLOCATE(AR2RHOU  )
    IF(ALLOCATED(ABRHOU   ))     DEALLOCATE(ABRHOU   )
    IF(ALLOCATED(TTRHOU   ))     DEALLOCATE(TTRHOU   )
    IF(ALLOCATED(SWLU     ))     DEALLOCATE(SWLU     )
    IF(ALLOCATED(PTSU     ))     DEALLOCATE(PTSU     )
    IF(ALLOCATED(PNSU     ))     DEALLOCATE(PNSU     )
    IF(ALLOCATED(PTSUA    ))     DEALLOCATE(PTSUA    )
    IF(ALLOCATED(PNSUA    ))     DEALLOCATE(PNSUA    )
    IF(ALLOCATED(RNU      ))     DEALLOCATE(RNU      )
    IF(ALLOCATED(RTU      ))     DEALLOCATE(RTU      )
    IF(ALLOCATED(PNBU     ))     DEALLOCATE(PNBU     )
    IF(ALLOCATED(PICU     ))     DEALLOCATE(PICU     )
    IF(ALLOCATED(SNBU     ))     DEALLOCATE(SNBU     )
    IF(ALLOCATED(RNU_ORG  ))     DEALLOCATE(RNU_ORG  )
    IF(ALLOCATED(PNSSO    ))     DEALLOCATE(PNSSO    )
    IF(ALLOCATED(PTSO     ))     DEALLOCATE(PTSO     )
    IF(ALLOCATED(PNSSAO   ))     DEALLOCATE(PNSSAO   )
    IF(ALLOCATED(PTSAO    ))     DEALLOCATE(PTSAO    )

    CALL DEALLOCATE_ERR_TRCOM1

    return

  END SUBROUTINE DEALLOCATE_ERR_TRCOMM

END MODULE TRCOMM
